@@ -1,6 +1,6 @@
 package Plerd;
 
-our $VERSION = '1.802';
+our $VERSION = '1.810';
 
 use Moose;
 use MooseX::Types::URI qw(Uri);
@@ -13,6 +13,7 @@ use Carp;
 use Try::Tiny;
 
 use Plerd::Post;
+use Plerd::Tag;
 use Plerd::WebmentionQueue;
 
 has 'path' => (
@@ -251,10 +252,9 @@ has 'has_tags' => (
 );
 
 has 'tags_map' => (
-    is => 'ro',
+    is => 'rw',
     isa => 'HashRef',
-    lazy_build => 1,
-    clearer => 'clear_tags_map',
+    default => sub { {} },
 );
 
 sub BUILD {
@@ -295,23 +295,8 @@ sub publish_all {
     $self->clear_posts;
     $self->clear_post_index_hash;
     $self->clear_post_url_index_hash;
-    $self->clear_tags_map;
-}
 
-# Return a structure of tags to post objects
-#  { TAG1 => [ post1, post2, ...],
-#    TAG2 => [],
-#    ...
-#  }
-sub _build_tags_map {
-    my $self = shift;
-    my %tags;
-    for my $post ( @{ $self->posts } ) {
-        for my $tag (@{$post->tags}) {
-            push @{ $tags{ $tag } }, $post;
-        }
-    }
-    return \%tags;
+    $self->tags_map( {} );
 }
 
 # Create a page that lists all available tags with
@@ -322,29 +307,39 @@ sub publish_tag_indexes {
 
     my $tag_map = $self->tags_map;
 
+    # Commentary: Ideally we'd just pass a sorted array of tag objects
+    # to the template. But alas said template was designed before tags were
+    # objects, and I didn't want to make Plerd users have to go mess around
+    # in their templates in between minor Plerd versions. And thus, we
+    # pull out some tag data into a hash and pass that in, instead.
+
     # Create all the individual tag pages
-    for my $tag (keys %$tag_map) {
+    for my $tag (values %$tag_map) {
 
         $self->template->process(
             $self->tags_template_file->open('<:encoding(utf8)'),
             {
-                self_uri => $self->tag_uri($tag),
+                self_uri => $tag->uri,
                 is_tags_page => 1,
-                tags => { $tag => $tag_map->{$tag} },
+                tags => { $tag->name => $tag->posts },
                 plerd => $self,
             },
-            $self->tags_publication_file($tag)->open('>:encoding(utf8)'),
+            $self->tags_publication_file($tag->name)->open('>:encoding(utf8)'),
             ) || $self->_throw_template_exception( $self->tags_template_file );
     }
 
     # Create the tag index
+    my %simplified_tag_map;
+    for my $tag (values %$tag_map) {
+        $simplified_tag_map{ $tag->name } = $tag->posts;
+    }
     $self->template->process(
         $self->tags_template_file->open('<:encoding(utf8)'),
         {
-            self_uri => $self->tag_uri,
+            self_uri => $self->tags_index_uri,
             is_tags_index_page => 1,
             is_tags_page => 1,
-            tags => $tag_map,
+            tags => \%simplified_tag_map,
             plerd => $self,
         },
         $self->tags_publication_file->open('>:encoding(utf8)'),
@@ -672,7 +667,10 @@ sub generates_post_guids {
 # Tag-related builders & methods
 sub _build_tags_index_uri {
     my $self = shift;
-    return $self->tag_uri;
+    return URI->new_abs(
+        'tags/',
+        $self->base_uri,
+    );
 }
 
 sub _build_tags_publication_path { 'tags' }
@@ -724,27 +722,46 @@ sub tags_publication_file {
     return $file;
 }
 
-sub tag_uri {
-    my ($self, $tag) = @_;
-    my $uri = $self->base_uri->clone;
-    unless (defined $tag) {
-        # master tag list
-        $uri->path($uri->path . $self->tags_publication_path . "/");
-        return $uri;
+sub tag_named {
+    my ( $self, $tag_name ) = @_;
+
+    my $key = lc $tag_name;
+
+    my $tag = $self->tags_map->{ $key };
+
+    if ( $tag ) {
+        $tag->ponder_new_name( $tag_name );
+    }
+    else {
+        $tag = Plerd::Tag->new(
+            name => $tag_name,
+            plerd => $self,
+        );
+        $self->tags_map->{ $key } = $tag;
     }
 
-    # individual tag page
-    $uri->path($uri->path . $self->tags_publication_path . "/$tag.html");
-    return $uri;
+    return $tag;
 }
 
-
 sub publish {
     my $self = shift;
 
     return $self->publish_all;
 }
 
+sub tag_uri {
+    my ( $self, $tag_name ) = @_;
+
+    my $tag = $self->tag_named( $tag_name );
+
+    if ( $tag ) {
+        return $tag->uri;
+    }
+    else {
+        return $self->tags_index_uri;
+    }
+}
+
 1;
 
 =encoding utf8
@@ -988,6 +1005,10 @@ Jason McIntosh <jmac@jmac.org>
 
 =item *
 
+Petter Hassberg
+
+=item *
+
 Joe Johnston
 
 =item *
