@@ -3,7 +3,7 @@ package Plerd::Post;
 use Moose;
 use DateTime;
 use DateTime::Format::W3CDTF;
-use Text::Markdown qw( markdown );
+use Text::MultiMarkdown qw( markdown );
 use URI;
 use HTML::Strip;
 use Data::GUID;
@@ -60,7 +60,7 @@ has 'attributes' => (
     isa => 'HashRef',
 );
 
-has 'tags' => (
+has 'tag_objects' => (
     is => 'rw',
     isa => 'ArrayRef',
     default => sub {[]}
@@ -488,9 +488,11 @@ sub _process_source_file {
     }
 
     if ( $attributes{ tags } ) {
-        my @tags = split /\s*,\s*/, $attributes{ tags };
-        if (@tags) {
-            @{ $self->tags } = @tags;
+        my @tag_names = split /\s*,\s*/, $attributes{ tags };
+        for my $tag_name (@tag_names) {
+            my $tag = $self->plerd->tag_named( $tag_name );
+            $tag->add_post( $self );
+            push @{ $self->tag_objects }, $tag;
         }
     }
 
@@ -788,6 +790,12 @@ sub _grep_webmentions {
     ];
 }
 
+sub tags {
+    my $self = shift;
+
+    return [ map { $_->name } @{ $self->tag_objects } ];
+}
+
 1;
 
 =head1 NAME
@@ -933,8 +941,15 @@ String representing this post's title.
 
 =item tags
 
-An array reference to the list of tags associated with this post as
-set in the source file using the 'tags:' header.
+An array reference to the list of tags (as plain-text strings)
+associated with this post as set in the source file using the 'tags:'
+header.
+
+=item tag_objects
+
+An array reference to the list of tags (as L<Plerd::Tag> objects)
+associated with this post as set in the source file using the 'tags:'
+header.
 
 =back
 
